@@ -11,10 +11,12 @@ public class GCubePortalConstants {
 	public static final String USER_MESSAGES_FRIENDLY_URL = "/messages";
 	public static final String VRES_EXPLORE_FRIENDLY_URL = "/explore";
 	public static final String CATALOGUE_FRIENDLY_URL = "/data-catalogue";
-	
+	public static final String AUTHORIZATION_FRIENDLY_URL = "/authorization";
 	
 	public static final String INFRASTRUCTURE_NAME = "infrastructure";  
 	public static final String SCOPES = "scopes";
+	public static final String PORTAL_APPLICATION_TOKEN = "portalAuthToken";  
+	public static final String ICPROXY_ENDPOINT_URL = "ic-proxy-endpointURL";  
 	public static final String SENDER_EMAIL = "notificationSenderEmail";
 	public static final String GATEWAY_NAME = "portalinstancename";
 	public static final String ADMIN_USERNAME = "administratorUsername";
