@@ -9,12 +9,17 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.Properties;
 
+import javax.portlet.RenderRequest;
 import javax.servlet.http.HttpServletRequest;
+import javax.servlet.http.HttpSession;
 
 import org.gcube.common.authorization.client.exceptions.ObjectNotFound;
 import org.gcube.common.authorization.library.provider.SecurityTokenProvider;
 import org.gcube.common.authorization.library.provider.UserInfo;
 import org.gcube.common.scope.api.ScopeProvider;
+import org.gcube.vomanagement.usermanagement.UserManager;
+import org.gcube.vomanagement.usermanagement.exception.UserManagementSystemException;
+import org.gcube.vomanagement.usermanagement.exception.UserRetrievalFault;
 import org.gcube.vomanagement.usermanagement.impl.LiferayGroupManager;
 import org.gcube.vomanagement.usermanagement.impl.LiferayUserManager;
 import org.gcube.vomanagement.usermanagement.model.CustomAttributeKeys;
@@ -35,9 +40,12 @@ import com.liferay.portal.util.PortalUtil;
 
 
 /**
+ * <p>
  * Clients can obtain the single instance of the {@link PortalContext} by invoking its static method {@link #getConfiguration()}. 
  * The first invocation of the method triggers the initialisation of the instance.
  * 
+ * For documentation see the related <a href="https://wiki.gcube-system.org/gcube/Portal_Context">WIKI</a>
+ * </p>
  * @author Massimiliano Assante (ISTI-CNR)
  *
  */
@@ -51,7 +59,6 @@ public class PortalContext {
 	protected static final String SCOPE_SEPARATOR = "/";
 
 	private final static String DEFAULT_ROLE = "OrganizationMember";
-
 	private static final String CONFIGURATION_FOLDER = "conf";
 	private static final String INFRA_PROPERTY_FILENAME = "infrastructure.properties";
 	private static final String GCUBE_DEV__CONTEXT_PROPERTY_FILENAME = "gcube-dev-context.properties";
@@ -75,6 +82,8 @@ public class PortalContext {
 
 	private static PortalContext singleton = new PortalContext();
 
+	private UserManager userManager;
+
 	private String infra;
 	private String vos;
 
@@ -103,10 +112,12 @@ public class PortalContext {
 			props.load( fis);
 			infra  = props.getProperty(GCubePortalConstants.INFRASTRUCTURE_NAME);
 			vos = props.getProperty(GCubePortalConstants.SCOPES);
+			userManager = new LiferayUserManager();
 		}
 		catch(IOException e) {
 			infra = DEFAULT_INFRA_NAME;
 			vos = DEFAULT_VO_NAME;
+			userManager = new LiferayUserManager();
 			_log.error("infrastructure.properties file not found under $CATALINA_HOME/conf/ dir, setting default infrastructure Name " + infra + " and VO Name " + vos);
 		}		
 
@@ -128,32 +139,65 @@ public class PortalContext {
 		return this.vos;
 	}
 	/**
+	 * <p>
+	 * Please note that this method works with AJAX calls only (i.e. XMLHttpRequest to exchange data with a server behind the scenes). 
+	 * If you use standard http servlet GET or POST to exchange data with a server, you must call the method PortalContext#setUserInSession in your portlet doView method.
 	 * 
-	 * @param httpServletRequest the httpServletRequest object
-	 * @return the instance of the current user 
-	 * @see GCubeUser
+	 * </p>
+	 * 
+	 * @param httpServletRequest the {@link HttpServletRequest} object
+	 * @return the current user, or <code>null</code> if a current user could not be found or session expired 
+	 * @see {@link GCubeUser}
 	 */
 	public GCubeUser getCurrentUser(HttpServletRequest httpServletRequest) {
-		Long userIdNo = ContextUserUtil.getCurrentUserId(httpServletRequest);
+		String userIdNo = httpServletRequest.getHeader(USER_ID_ATTR_NAME);
+		long userId = -1;
 		if (userIdNo != null) {
 			try {
-				return new LiferayUserManager().getUserById(userIdNo);
-			}  catch (Exception e) {
-				_log.error("The userId does not belong to any user -> " + userIdNo);
+				_log.debug("The userIdNo is " + userIdNo);
+				userId = Long.parseLong(userIdNo);
+				return userManager.getUserById(userId);
+			} catch (NumberFormatException e) {
+				_log.error("The userId is not a number -> " + userId);
+			} catch (Exception e) {
+				_log.error("Could not read the current userid from header param, either session expired or user not logged in, exception: " + e.getMessage());
 			}
 		} else {
-			if (isWithinPortal()) {
-				_log.warn("Could not read the current userid, either session expired or user not logged in");
-			} else {
+			if (!isWithinPortal()) {
 				GCubeUser toReturn = readUserFromPropertyFile();
 				_log.debug("getCurrentUser devMode into IDE detected, returning testing user: " + toReturn.toString());
 				return toReturn;
+			} else { //must be a custom servlet
+				try {
+					HttpSession session = httpServletRequest.getSession(false);
+					userId = (long) session.getAttribute(USER_ID_ATTR_NAME);
+					_log.debug("read the current userid from the http session userId: " + userId + " sessionid="+session.getId());
+					return userManager.getUserById(userId);
+				} catch (Exception e) {
+					_log.error("Could not read the current userid from the http session, either session expired or user not logged in, exception: " + e.getMessage());
+					return null;
+				}
 			}
 		}
 		return null;
 	}
 	/**
+	 * <p>
+	 * This method sets the current userid in the session so that it is possible to get the current user in custom servlets of the same WAR. 
+	 * Must be added in Portlet's doView() method before dispatching the request.
+	 * </p>
 	 * 
+	 * @param request the portlet {@link RenderRequest} object
+	 */
+	public static void setUserInSession(RenderRequest request) {
+		long userId = Long.parseLong(request.getRemoteUser());
+		
+		HttpServletRequest httpServletRequest = PortalUtil.getHttpServletRequest(request);
+		HttpSession session = httpServletRequest.getSession();
+		_log.debug("HttpSession#setUserInSession, set userId: " + userId + " sessionid="+session.getId());
+		session.setAttribute(USER_ID_ATTR_NAME, userId);
+	}
+	/**
 	 * @param scopeGroupId the liferay groupid number (as String) of the VRE/VO
 	 * @return the scope (context)
 	 */
@@ -183,19 +227,28 @@ public class PortalContext {
 		return null;
 	}
 	/**
+	 * <p>
+	 * Please note that this method works with AJAX calls only (i.e. XMLHttpRequest to exchange data with a server behind the scenes). 
+	 * If you use standard http servlet GET or POST to exchange data with a server, you must you must handle the infrastructure context information differently. 
+	 * Please see the following page for further information  @see <a href="https://wiki.gcube-system.org/gcube/ClientContextLibrary">ClientContextLibrary WIKI</a>
+	 * </p>	
 	 * 
-	 * @param httpServletRequest the httpServletRequest object
-	 * @return the scope (context)
+	 * @param httpServletRequest the {@link HttpServletRequest} object
+	 * @return the infrastructure context (scope)
 	 */
 	public String getCurrentScope(HttpServletRequest httpServletRequest) {
 		String scopeGroupId = httpServletRequest.getHeader(VRE_ID_ATTR_NAME);
 		return getCurrentScope(scopeGroupId);
 	} 
 	/**
+	 * <p>
+	 * Please note that this method works with AJAX calls only (i.e. XMLHttpRequest to exchange data with a server behind the scenes). 
+	 * If you use standard http servlet GET or POST to exchange data with a server, you must you must handle the infrastructure context information differently. 
+	 * Please see the following page for further information  @see <a href="https://wiki.gcube-system.org/gcube/ClientContextLibrary">ClientContextLibrary WIKI</a>
+	 * </p>	
 	 * 
-	 * @param httpServletRequest the httpServletRequest object
-	 * @return the instance of the user 
-	 * @see GCubeUser
+	 * @param httpServletRequest the {@link HttpServletRequest} object
+	 * @return the current group name (e.g. devVRE, BioDiversityLab, RStudioLab etc. )
 	 */
 	public String getCurrentGroupName(HttpServletRequest httpServletRequest) {
 		String groupIdNo = httpServletRequest.getHeader(VRE_ID_ATTR_NAME);
@@ -221,10 +274,14 @@ public class PortalContext {
 		return null;
 	}
 	/**
+	 * <p>
+	 * Please note that this method works with AJAX calls only (i.e. XMLHttpRequest to exchange data with a server behind the scenes). 
+	 * If you use standard http servlet GET or POST to exchange data with a server, you must you must handle the infrastructure context information differently. 
+	 * Please see the following page for further information  @see <a href="https://wiki.gcube-system.org/gcube/ClientContextLibrary">ClientContextLibrary WIKI</a>
+	 * </p>	
 	 * 
-	 * @param httpServletRequest the httpServletRequest object
-	 * @return the instance of the user 
-	 * @see GCubeUser
+	 * @param httpServletRequest the {@link HttpServletRequest} object
+	 * @return the current group identifier as long
 	 */
 	public long getCurrentGroupId(HttpServletRequest httpServletRequest) {
 		String groupIdNo = httpServletRequest.getHeader(VRE_ID_ATTR_NAME);
@@ -248,13 +305,73 @@ public class PortalContext {
 		}
 		return -1;
 	}
-
-
 	/**
-	 * @deprecated use only with AJAX Calls
-	 * @param httpServletRequest the httpServletRequest object
-	 * @return the instance of the user 
-	 * @see GCubeUser
+	 * <p>
+	 * Returns the gCube authorisation token for the given user 
+	 * </p>	
+	 * @param scope infrastrucure context (scope)
+	 * @param userId the GCubeUser user identifier (userId) @see {@link GCubeUser}
+	 * @return the Token for the user in the context, or <code>null</code> if a token for this user could not be found
+	 */
+	public String getCurrentUserToken(String scope, long userId) {
+		if (isWithinPortal()) {
+			try {
+				String username = userManager.getUserById(userId).getUsername();
+				return  getCurrentUserToken(scope, username);
+			} catch (UserManagementSystemException | UserRetrievalFault e) {
+				e.printStackTrace();
+			}
+		}
+		else {
+			String toReturn = readTokenPropertyFile();
+			_log.debug("getCurrentToken devMode into IDE detected, returning scope: " + toReturn.toString());
+			_log.debug("The PortalBeanLocatorUtil stacktrace (java.lang.Exception) is acceptable in dev");
+			return toReturn;
+		}
+		return null;
+	}
+	/**
+	 * <p>
+	 * Returns the gCube authorisation token for the given user 
+	 * </p>	
+	 * @param scope infrastrucure context (scope)
+	 * @param username the GCubeUser username @see {@link GCubeUser}
+	 * @return the Token for the user in the context, or <code>null</code> if a token for this user could not be found
+	 */
+	public String getCurrentUserToken(String scope, String username) {
+		String userToken = null;
+		if (isWithinPortal()) {
+			try {
+				ScopeProvider.instance.set(scope);
+				userToken = authorizationService().resolveTokenByUserAndContext(username, scope);
+				SecurityTokenProvider.instance.set(userToken);
+			} 
+			catch (ObjectNotFound ex) {
+				userToken = generateAuthorizationToken(username, scope);
+				SecurityTokenProvider.instance.set(userToken);
+				_log.debug("generateAuthorizationToken OK for " + username + " in scope " + scope);
+			}			 
+			catch (Exception e) {
+				_log.error("Error while trying to generate token for user " + username + "in scope " + scope);
+				e.printStackTrace();
+				return null;
+			}
+		} else {
+			String toReturn = readTokenPropertyFile();
+			_log.debug("getCurrentToken devMode into IDE detected, returning scope: " + toReturn.toString());
+			_log.debug("The PortalBeanLocatorUtil stacktrace (java.lang.Exception) is acceptable in dev");
+			return toReturn;
+		}
+		return userToken;
+	}
+	/**
+	 * @deprecated please use getCurrentUserToken(String scope, String username) or getCurrentUserToken(String scope, long userId)
+	 * <p>
+	 * Please note that this method works with AJAX calls only (i.e. XMLHttpRequest to exchange data with a server behind the scenes). 
+	 * </p>	
+	 * 
+	 * @param httpServletRequest the {@link HttpServletRequest} object
+	 * @return the Token for the user in the context, or <code>null</code> if a token for this user could not be found
 	 */
 	public String getCurrentUserToken(HttpServletRequest httpServletRequest) {
 		String groupIdNo = httpServletRequest.getHeader(VRE_ID_ATTR_NAME);
@@ -289,40 +406,6 @@ public class PortalContext {
 		}
 		return userToken;
 	}
-	/**
-	 * @param httpServletRequest the httpServletRequest object
-	 * @param scope the infrastructure scope
-	 * @return the instance of the user 
-	 * @see GCubeUser
-	 */
-	public String getCurrentUserToken(String scope, HttpServletRequest httpServletRequest) {
-		String userToken = null;
-		if (isWithinPortal()) {
-			String username = getCurrentUser(httpServletRequest).getUsername();
-			try {
-				ScopeProvider.instance.set(scope);
-				userToken = authorizationService().resolveTokenByUserAndContext(username, scope);
-				SecurityTokenProvider.instance.set(userToken);
-			} 
-			catch (ObjectNotFound ex) {
-				userToken = generateAuthorizationToken(username, scope);
-				SecurityTokenProvider.instance.set(userToken);
-				_log.debug("generateAuthorizationToken OK for " + username + " in scope " + scope);
-			}			 
-			catch (Exception e) {
-				_log.error("Error while trying to generate token for user " + username + "in scope " + scope);
-				e.printStackTrace();
-				return null;
-			}
-		} else {
-			String toReturn = readTokenPropertyFile();
-			_log.debug("getCurrentToken devMode into IDE detected, returning scope: " + toReturn.toString());
-			_log.debug("The PortalBeanLocatorUtil stacktrace (java.lang.Exception) is acceptable in dev");
-			return toReturn;
-		}
-		return userToken;
-	}
-
 	/**
 	 * 
 	 * @param username
@@ -381,7 +464,7 @@ public class PortalContext {
 	}
 	/**
 	 * 
-	 * @param httpServletRequest
+	 * @param httpServletRequest the {@link HttpServletRequest} object
 	 * @return the gateway URL until the first slash, e.g. http(s)://mynode.d4science.org:8080, if the URL uses standard http(s) port like 80 or 443 the port is not returned. 
 	 */
 	public String getGatewayURL(HttpServletRequest httpServletRequest) {
@@ -425,7 +508,7 @@ public class PortalContext {
 	}
 	/**
 	 * 
-	 * @param request
+	 * @param httpServletRequest the {@link HttpServletRequest} object
 	 * @return the landing page path of the current Site e.g. "/group/i-marine"
 	 */
 	public String getSiteLandingPagePath(final HttpServletRequest request) {
@@ -465,7 +548,7 @@ public class PortalContext {
 	}
 	/**
 	 * 
-	 * @param request
+	 * @param request the {@link HttpServletRequest} object
 	 * @return the current Group instance based on the request
 	 */
 	private Group getSiteFromServletRequest(final HttpServletRequest request) throws Exception {
@@ -507,11 +590,8 @@ public class PortalContext {
 		return null;
 	}
 	/**
-	 * @param request
 	 * @param currentGroup
-	 * @param isPrivate
-	 * @param isUser
-	 * @return
+	 * @return Returns the friendly u r l of this group.
 	 */
 	private static String getGroupFriendlyURL(final Group currentGroup) throws Exception {
 		String friendlyURL = GCubePortalConstants.PREFIX_GROUP_URL;
@@ -537,7 +617,7 @@ public class PortalContext {
 	}
 	/**
 	 * 
-	 * @param request the HttpServletRequest instance of your servlet
+	 * @param httpServletRequest the {@link HttpServletRequest} object
 	 * @return the current Site Name based on the request
 	 */
 	public String getGatewayName(HttpServletRequest request) {
@@ -580,7 +660,7 @@ public class PortalContext {
 	}
 	/**
 	 * 
-	 * @param request the HttpServletRequest instance of your servlet
+	 * @param request the {@link HttpServletRequest} object
 	 * @return the sender (from) email address  for the current Site based on the request
 	 */
 	public String getSenderEmail(HttpServletRequest request) {
@@ -611,6 +691,59 @@ public class PortalContext {
 		return toReturn;
 	}
 
+	/**
+	 * <p>
+	 *  read the portal application token valid in the root Virtual Organisation into a property file
+	 * </p>
+	 * @return the portal application token valid in the root Virtual Organisation or <code>null</code> if the token could not be found
+	 */
+	public static String getPortalApplicationToken() {
+		//get the portles to look for from the property file
+		Properties props = new Properties();
+		String toReturn = "";
+
+		try {
+			String propertyfile = getCatalinaHome() + File.separator + "conf" + File.separator + "gcube-data.properties";			
+			File propsFile = new File(propertyfile);
+			FileInputStream fis = new FileInputStream(propsFile);
+			props.load( fis);
+			toReturn = props.getProperty(GCubePortalConstants.PORTAL_APPLICATION_TOKEN);
+		}
+		//catch exception in case properties file does not exist
+		catch(IOException e) {
+			toReturn = null;
+			_log.error("gcube-data.properties file or portal app token not found under $CATALINA_HOME/conf dir, please add the property and or a valid token: " + GCubePortalConstants.PORTAL_APPLICATION_TOKEN);
+			return toReturn;
+		}
+		return toReturn;
+	}
+	/**
+	 * <p>
+	 *  read the IC Proxy endpoint from a property file, IC Proxy is needed to query the Information System via REST
+	 *  e.g. http://dev.d4science.org:8080/icproxy/gcube/service
+	 * </p>
+	 * @return the IC Proxy endpoint or <code>null</code> if the IC Proxy could not be found
+	 */
+	public static String getICProxyEndPoint() {
+		//get the portles to look for from the property file
+		Properties props = new Properties();
+		String toReturn = "";
+
+		try {
+			String propertyfile = getCatalinaHome() + File.separator + "conf" + File.separator + "gcube-data.properties";			
+			File propsFile = new File(propertyfile);
+			FileInputStream fis = new FileInputStream(propsFile);
+			props.load( fis);
+			toReturn = props.getProperty(GCubePortalConstants.ICPROXY_ENDPOINT_URL);
+		}
+		//catch exception in case properties file does not exist
+		catch(IOException e) {
+			toReturn = null;
+			_log.error("gcube-data.properties file or portal app token not found under $CATALINA_HOME/conf dir, please add the property and or a valid token: " + GCubePortalConstants.PORTAL_APPLICATION_TOKEN);
+			return toReturn;
+		}
+		return toReturn;
+	}
 
 	/**
 	 * read the sender (from) email address for notifications name from a property file and returns it
@@ -785,6 +918,8 @@ public class PortalContext {
 		}		
 		return -1L;
 	}
+	
+	
 
 
 	/**
